/*

  KanjiDraw.java
  KanjiDraw

 Created by Julio Gorge on 07/11/2006.
 Copyright 2006-2007 LemonTeam.
 
 Kana DS is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Kana DS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Kana DS; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

import java.awt.geom.Point2D;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class KanjiDraw extends Applet implements ActionListener
{
	// Member variables
	
	String message = "";
	
	boolean gesturesMatch = false;
	
	boolean drawingGesture = false;
	boolean drawingStroke = false;
	
	boolean isMouseDown = false;
	
	Gesture theGesture;
	Stroke theStroke;
	
	Gesture storedGesture = null;
	Gesture storedGestureNotNormalized = null;
	
	
	// App UI layout dimensions
	
	static final int sBorderSize = 3;
	static final int sDrawingAreaX0 = sBorderSize;
	static final int sDrawingAreaY0 = sBorderSize;
	static final int sDrawingAreaX1 = 422;
	static final int sDrawingAreaY1 = 378;
	static final int kDrawingAreaBorderX = sDrawingAreaX0 + sDrawingAreaX1;

	Font font;
	Color backgroundColor;
	
	
	
	// Initialize applet
	
    public void init()
	{
		font = new Font( "helvetica", Font.BOLD, 14 );
		backgroundColor = new Color( 1.0f, 1.0f, 0.9f );
		
		theGesture = new Gesture();
		theStroke = new Stroke();				
    }
	
	
	// Handle action events
	
	public void actionPerformed( ActionEvent evt )
	{
		repaint();
	}
	
	
	// Handle paint event
	
    public void paint (Graphics g)
	{
		Graphics2D g2d = (Graphics2D)g;
		
		setBackground( backgroundColor );
			
		repaintDrawingArea( g );
		drawButtons( g );
		
		theGesture.draw( g2d );
		
		if ( storedGestureNotNormalized != null )
			storedGestureNotNormalized.draw( g2d, 440, 28, 0.13f, 0.13f );
		
		if ( drawingGesture && drawingStroke )
			theStroke.draw( g2d );
		
        g.setColor(Color.black);
        g.setFont(font);
        g.drawString( message, 10, 20 );

    }
	
	
	// Clear drawing area
	
	private void repaintDrawingArea( Graphics g )
	{
		g.setColor( backgroundColor );
		g.fillRect( sDrawingAreaX0, sDrawingAreaY0, sDrawingAreaX1, sDrawingAreaY1 );
		g.setColor( Color.black );
		g.drawRect( sDrawingAreaX0, sDrawingAreaY0, sDrawingAreaX1, sDrawingAreaY1 ); 
	}
	
	
	// Draw the buttons at the right side of the screen
	
	private void drawButtons( Graphics g )
	{
		// "STORE GESTURE" button
		g.setColor( Color.lightGray );
		g.fillRect( kDrawingAreaBorderX + sDrawingAreaX0, sDrawingAreaY0,
					getWidth() - ( kDrawingAreaBorderX + sBorderSize * 2 ), 75 );
					
		g.setColor( Color.black );
		g.drawRect( kDrawingAreaBorderX + sDrawingAreaX0, sDrawingAreaY0,
					getWidth() - ( kDrawingAreaBorderX + sBorderSize * 2 ), 75 );
										
		// "ERASE SCREEN" button			
		g.setColor( Color.lightGray );
		g.fillRect( kDrawingAreaBorderX + sDrawingAreaX0, sDrawingAreaY0 + 75 + sBorderSize,
					getWidth() - ( kDrawingAreaBorderX + sBorderSize * 2 ), 75 );
					
		g.setColor( Color.black );
		g.drawRect( kDrawingAreaBorderX + sDrawingAreaX0, sDrawingAreaY0 + 75 + sBorderSize,
					getWidth() - ( kDrawingAreaBorderX + sBorderSize * 2 ), 75);		
	
		// "MATCH?" button			
		g.setColor( Color.lightGray );
		g.fillRect( kDrawingAreaBorderX + sDrawingAreaX0, sDrawingAreaY0 + 150 + sBorderSize * 2,
					getWidth() - ( kDrawingAreaBorderX + sBorderSize * 2 ), 75 );
		
		g.setColor( Color.black );
		g.drawRect( kDrawingAreaBorderX + sDrawingAreaX0, sDrawingAreaY0 + 150 + sBorderSize * 2,
					getWidth() - ( kDrawingAreaBorderX + sBorderSize * 2 ), 75);		
	
	
							
		// Draw text messages
		
		g.setFont(font);
		g.drawString( "set gesture", sDrawingAreaX1 + 8, 22 );
		g.drawString( "erase", sDrawingAreaX1 + 8, 35 + 62 );
		g.drawString( "match?", sDrawingAreaX1 + 8, 35 + 142 );
		
		if( gesturesMatch )
		{
			g.setColor( Color.green );
			g.drawString( "yes", sDrawingAreaX1 + 8, 35 + 165 ); 
		}
		else
		{
			g.setColor( Color.red );
			g.drawString( "no", sDrawingAreaX1 + 8, 35 + 165 ); 
		}

	}
	
	
	// Overridden methods for event handling

	public boolean mouseDrag( Event evt, int x, int y)
	{	
		if ( drawingGesture && drawingStroke && x > 8 && x < sDrawingAreaX1 - 8  && y > 8 && y < sDrawingAreaY1 - 8 )
		{
			Point2D.Float pt = new Point2D.Float( (float)x, (float)y );
		
			theStroke.add( pt );
			
			repaint();
		}
		
		return true;
	}

	public boolean mouseDown( Event evt, int x, int y)
	{
		isMouseDown = true;
		message = "";
		gesturesMatch = false;
		
		if ( x < sDrawingAreaX1 )
		{
			if ( !drawingGesture )
			{
				drawingGesture = true;
				drawingStroke = true;
				
				theGesture.clear();
				theStroke.clear();
			}
			else if ( !drawingStroke )
			{
				theStroke.clear();
				drawingStroke = true;
			}
		}
		else
		{
			if ( y < 75 )
			{							
				storedGesture = (Gesture)theGesture.clone();
				storedGestureNotNormalized = (Gesture)theGesture.clone();
				
				theGesture.clear();
				drawingGesture = false;
				drawingStroke = false;
				
				message = "gesture stored!";
			}
			else if ( y < 75 * 2 + 3 )
			{
				theGesture.clear();
				drawingGesture = false;
				drawingStroke = false;
				
				message = "screen erased!";
			}
			
		}
		
		repaint();
		
		return true;
	}
	
	public boolean mouseUp( Event evt, int x, int y)
	{
		isMouseDown = false;
		
		if ( drawingGesture )
		{
			if ( drawingStroke )
			{
				drawingStroke = false;
				
				Stroke stroke = (Stroke)theStroke.clone();
				
				// add stroke to gesture
				theGesture.addStroke( stroke );
				theStroke.clear();
				
				performMatch();
			}
		}
		

		repaint();

		return true;
	}
	
	
	// Compare currently displayed gesture with the one that was previously stored in memory
	
	private void performMatch()
	{	
		if ( storedGesture != null )
		{
			Gesture comparingGesture = (Gesture)theGesture.clone();
					
			float factor = storedGesture.match( comparingGesture );
					
			message = "matching factor: " + factor;
					
			if ( factor >= 0.98f )
				gesturesMatch = true;
			else
				gesturesMatch = false;
		}
	}
}
